@@ -4,7 +4,7 @@
 
   Author: Giovanni Giacobbi <giovanni@giacobbi.net>
 
-  Copyright (C) 1997 - 2003 Pekka Riikonen
+  Copyright (C) 1997 - 2004 Pekka Riikonen
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
   GNU General Public License for more details.
 
 */
-/* $Id: serverconfig.c,v 1.68.2.12 2003/11/08 11:10:05 priikone Exp $ */
+/* $Id: serverconfig.c,v 1.68.2.16 2005/03/27 16:12:11 priikone Exp $ */
 
 #include "serverincludes.h"
 #include "server_internal.h"
@@ -127,9 +127,9 @@ static bool my_parse_authdata(SilcAuthMethod auth_meth, const char *p,
     if (auth_data && auth_data_len) {
       if (!silc_utf8_valid(p, strlen(p))) {
 	*auth_data_len = silc_utf8_encoded_len(p, strlen(p),
-					       SILC_STRING_LANGUAGE);
+					       SILC_STRING_LOCALE);
 	*auth_data = silc_calloc(*auth_data_len, sizeof(unsigned char));
-	silc_utf8_encode(p, strlen(p), SILC_STRING_LANGUAGE, *auth_data,
+	silc_utf8_encode(p, strlen(p), SILC_STRING_LOCALE, *auth_data,
 			 *auth_data_len);
       } else {
 	*auth_data = (void *) strdup(p);
@@ -730,7 +730,10 @@ SILC_CONFIG_CALLBACK(fetch_connparam)
     config->tmp = NULL;
     return SILC_CONFIG_OK;
   }
-  SILC_SERVER_CONFIG_ALLOCTMP(SilcServerConfigConnParams);
+  if (!tmp) {
+    SILC_SERVER_CONFIG_ALLOCTMP(SilcServerConfigConnParams);
+    tmp->reconnect_keep_trying = TRUE;
+  }
 
   if (!strcmp(name, "name")) {
     CONFIG_IS_DOUBLE(tmp->name);
@@ -1258,11 +1261,11 @@ static const SilcConfigTable table_connparam[] = {
   { "version_software",	       SILC_CONFIG_ARG_STR,    fetch_connparam,	NULL },
   { "version_software_vendor", SILC_CONFIG_ARG_STR,    fetch_connparam,	NULL },
   { "anonymous",               SILC_CONFIG_ARG_TOGGLE, fetch_connparam,	NULL },
-  { "qos",    	               SILC_CONFIG_ARG_TOGGLE,	fetch_generic,	NULL },
-  { "qos_rate_limit",          SILC_CONFIG_ARG_INT,	fetch_generic,	NULL },
-  { "qos_bytes_limit",         SILC_CONFIG_ARG_INT,	fetch_generic,	NULL },
-  { "qos_limit_sec",           SILC_CONFIG_ARG_INT,	fetch_generic,	NULL },
-  { "qos_limit_usec",          SILC_CONFIG_ARG_INT,	fetch_generic,	NULL },
+  { "qos",    	               SILC_CONFIG_ARG_TOGGLE, fetch_connparam,	NULL },
+  { "qos_rate_limit",          SILC_CONFIG_ARG_INT,    fetch_connparam,	NULL },
+  { "qos_bytes_limit",         SILC_CONFIG_ARG_INT,    fetch_connparam,	NULL },
+  { "qos_limit_sec",           SILC_CONFIG_ARG_INT,    fetch_connparam,	NULL },
+  { "qos_limit_usec",          SILC_CONFIG_ARG_INT,    fetch_connparam,	NULL },
   { 0, 0, 0, 0 }
 };
 
@@ -1401,6 +1404,12 @@ static bool silc_server_config_check(SilcServerConfig config)
 	  "same host.", r->host));
       ret = FALSE;
     }
+
+    if (r->initiator == FALSE && r->port != 0) {
+      SILC_SERVER_LOG_WARNING(("\nWarning: Initiator is FALSE and Port is "
+                               "specified. Ignoring Port value."));
+      r->port = 0;
+    }
   }
 
   /* ServerConnection sanity checks */
@@ -1448,6 +1457,7 @@ SilcServerConfig silc_server_config_alloc(const char *filename)
   /* general config defaults */
   config_new->refcount = 1;
   config_new->logging_timestamp = TRUE;
+  config_new->param.reconnect_keep_trying = TRUE;
 
   /* obtain a config file object */
   file = silc_config_open(filename);
